﻿
Imports System.IO.Ports

Public Class Form_Main
    Inherits System.Windows.Forms.Form

    Private EventsAreEnabled As Boolean = False
    Private WithEvents comPort As New SerialPort()
    Private PortNumber As Int32
    Private BaudRate As Int32
    Private Slot As Int32
    Private Protocol As Int32
    Private Counter As Int32 = -1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        EventsAreEnabled = False
        '
        Me.Text = AppTitleAndVersion("Theremino Bridge")
        '
        MemoryMappedFile_Init()
        ' --------------------------------------- default PortNumber, BaudRate and slot
        cmb_PortNumber.SelectedIndex = 1
        cmb_BaudRate.SelectedIndex = 4
        txt_Slot.Text = "0"
        ' ---------------------------------------
        Load_INI()
        EnableDisableLog()
        ' --------------------------------------- Set the combo with the port number
        SetParams()
        ' --------------------------------------- Auto-connect at start
        Connect_ComPort()
        '
        EventsAreEnabled = True
    End Sub

    Private Sub MainForm_FormClosed(ByVal eventSender As System.Object, _
                                    ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) _
                                    Handles Me.FormClosed
        Disconnect_ComPort()
        Save_INI()
    End Sub


    ' ================================================================================================
    '  Controls
    ' ================================================================================================
    Private Sub cmd_Connect_Click(ByVal eventSender As System.Object, _
                                  ByVal eventArgs As System.EventArgs) Handles cmd_Connect.Click
        cmd_Connect.Visible = True
        cmd_Disconnect.Visible = False
        Connect_ComPort()
    End Sub
    Private Sub cmd_Disconnect_Click(ByVal eventSender As System.Object, _
                                     ByVal eventArgs As System.EventArgs) Handles cmd_Disconnect.Click
        cmd_Connect.Visible = True
        cmd_Disconnect.Visible = False
        Disconnect_ComPort()
    End Sub
    Private Sub cmb_PortNumber_SelectedIndexChanged(ByVal sender As System.Object, _
                                                    ByVal e As System.EventArgs) Handles cmb_PortNumber.SelectedIndexChanged
        If Not EventsAreEnabled Then Return
        SetParams()
        Connect_ComPort()
        Save_INI()
    End Sub
    Private Sub cmb_BaudRate_SelectedIndexChanged(ByVal sender As System.Object, _
                                                    ByVal e As System.EventArgs) Handles cmb_BaudRate.SelectedIndexChanged
        If Not EventsAreEnabled Then Return
        SetParams()
        Connect_ComPort()
        Save_INI()
    End Sub
    Private Sub txt_Slot_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Slot.TextChanged
        If Not EventsAreEnabled Then Return
        SetParams()
        Save_INI()
    End Sub
    Private Sub chk_ShowLog_CheckedChanged(ByVal sender As System.Object, _
                                           ByVal e As System.EventArgs) Handles chk_ShowLog.CheckedChanged
        If Not EventsAreEnabled Then Return
        EnableDisableLog()
    End Sub
    Private Sub EnableDisableLog()
        Dim topSize As Int32 = Me.Size.Height - Me.ClientSize.Height
        If chk_ShowLog.Checked Then
            Me.FormBorderStyle = Windows.Forms.FormBorderStyle.Sizable
            Me.MinimumSize = New Size(TextBox2.Right + 12, 680 + topSize)
            'Me.MinimumSize = New Size(684, 320)
            Me.Size = Me.MinimumSize
        Else
            Me.FormBorderStyle = Windows.Forms.FormBorderStyle.FixedSingle
            Me.MinimumSize = New Size(txt_Counts.Right + 12, txt_Counts.Bottom + topSize + 6)
            'Me.MinimumSize = New Size(684, 98)
            Me.Size = Me.MinimumSize
        End If
        TextBox2.Text = ""
        TextBox1.Text = ""
    End Sub

    Private Sub SetParams()
        If cmb_PortNumber.SelectedIndex < 0 Then cmb_PortNumber.SelectedIndex = 0
        If cmb_BaudRate.SelectedIndex < 0 Then cmb_BaudRate.SelectedIndex = 6
        If cmb_Protocol.SelectedIndex < 0 Then cmb_Protocol.SelectedIndex = 0
        PortNumber = CInt(Val(cmb_PortNumber.Text))
        BaudRate = CInt(Val(cmb_BaudRate.Text))
        Slot = CInt(Val(txt_Slot.Text))
        Protocol = CInt(Val(cmb_Protocol.Text))
    End Sub


    ' ================================================================================================
    '  Connect - Disconnect - Receive
    ' ================================================================================================
    Private Sub Connect_ComPort()
        ' -------------------------------------------- If the port is Open then close it
        Disconnect_ComPort()
        ' -------------------------------------------- Set PORT and SPEED
        comPort.PortName = "COM" & PortNumber.ToString
        comPort.BaudRate = BaudRate
        comPort.Parity = Parity.None
        comPort.DataBits = 8
        comPort.StopBits = StopBits.One
        ' -------------------------------------------- Set params
        comPort.ReceivedBytesThreshold = 1
        comPort.ReadBufferSize = 5000
        ' -------------------------------------------- Disable DTR
        comPort.DtrEnable = False
        ' -------------------------------------------- Try to open the port
        Try
            comPort.Open()
        Catch
        End Try
        ' -------------------------------------------- Test if the port is really open
        If comPort.IsOpen Then
            cmd_Disconnect.Visible = True
            cmd_Connect.Visible = False
        Else
            cmd_Disconnect.Visible = False
            cmd_Connect.Visible = True
        End If
    End Sub

    Private Sub Disconnect_ComPort()
        If comPort.IsOpen Then
            comPort.Close()
        End If
    End Sub


    ' ================================================================================================
    '  Receive from serial
    ' ================================================================================================
    Private Sub comPort_DataReceived(ByVal sender As Object, _
                                     ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) _
                                     Handles comPort.DataReceived

        ' ================================================ decode arrived data 
        Dim s As String = comPort.ReadExisting
        Select Case Protocol
            Case 0
                ' ---------------------------------------- RAD protocol - 1 count for each serial character
                Counter += s.Length
            Case 1
                ' ---------------------------------------- pre-incremented 16 bit data - ending with LineFeed
                If s.EndsWith(vbLf) Then
                    Counter = CInt(Val(s))
                End If
            Case 2
                ' ---------------------------------------- incremental data - ending with LineFeed
                If s.EndsWith(vbLf) Then
                    Counter += CInt(Val(s))
                End If
        End Select
        ' ------------------------------------------------ limit counter
        If Counter > 65535 Then Counter -= 65536
        ' ================================================ send data to Theremino Slot
        If Slot >= 0 Then
            ' -------------------------------------------- init counter
            If Counter < 0 Then
                Counter = CInt(ReadSlot_NoNan(Slot))
            End If
            ' -------------------------------------------- send counter
            WriteSlot(Slot, Counter)
        End If
        ' ================================================ visualize the Counter
        txt_Counts.Text = Counter.ToString
        ' ================================================ visualize ASCII and HEX TextBoxes
        If chk_ShowLog.Checked Then
            Me.Invoke(New Action(Of String)(AddressOf UpdateTextBox), s)
        End If
    End Sub

    Private Sub UpdateTextBox(ByVal s As String)
        TextBox2.Text &= s.Replace(vbCr, ".")
        For i As Int32 = 1 To s.Length
            TextBox1.Text &= Asc(Mid(s, i, 1)).ToString("00") & " "
        Next
    End Sub

End Class